package de.hamster.workbench;

import java.awt.Color;
import java.awt.Component;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.awt.event.KeyEvent;
import java.awt.event.InputEvent;
import java.io.File;
import java.io.FileInputStream;
import java.net.URL;
import java.text.MessageFormat;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.ResourceBundle;

import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.attribute.standard.JobName;
import javax.print.attribute.standard.OrientationRequested;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JToggleButton;
import javax.swing.KeyStroke;

import de.hamster.editor.view.TextAreaPrintable;

/**
 * Diese Klasse enthaelt Hilfsmethoden und Konstanten, die im Hamster-Simulator
 * benoetigt werden.
 * 
 * @author Daniel Jasper
 */
public class Utils {
	public static final int YES = JOptionPane.YES_OPTION;

	public static final int NO = JOptionPane.NO_OPTION;

	public static final int CANCEL = JOptionPane.CANCEL_OPTION;

	public static final int TYPE_HAM = 1;

	public static final int TYPE_TER = 2;

	public static final String PSEP = System.getProperty("path.separator");

	public static final String FSEP = System.getProperty("file.separator");

	public static final String LSEP = System.getProperty("line.separator");

	// TODO: Umlegen von HOME ins aktuelle Verzeichnis
	public static String HOME = System.getProperty("user.dir") + FSEP
			+ "Programme";

	public static final Color[] COLORS = { Color.BLUE, Color.RED, Color.GREEN,
			Color.YELLOW, Color.CYAN, Color.MAGENTA, Color.WHITE };

	/**
	 * Der Rand von Buttons in der Toolbar. Noetig zur Einhaltung der Java Look
	 * and Feel Design Guidelines.
	 */
	public static final Insets TOOLBAR_MARGIN = new Insets(0, 0, 0, 0);

	/**
	 * Das geladene ResourceBundle, passend zur aktuellen Locale
	 */
	private static ResourceBundle resources;

	/**
	 * Mit dieser Methode kann das ResourceBundle erfragt werden. Ist es noch
	 * nicht geladen, wird es aus der entsprechenden Datei gelesen.
	 * 
	 * @return Das ResourceBundle
	 */
	public static ResourceBundle getResources() {
		if (resources == null)
			resources = ResourceBundle.getBundle("resources.hamster");
		return resources;
	}

	public static String getResource(String key) {
		return getResources().getString(key);
	}

	public static String getResource(String key, String param) {
		return MessageFormat.format(getResource(key), new Object[] { param });
	}

	public static void print(String text) {
		PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();
		aset.add(OrientationRequested.LANDSCAPE);
		aset.add(new JobName("Hamster-Programm", null));

		/* Create a print job */
		PrinterJob pj = PrinterJob.getPrinterJob();
		pj.setPrintable(new TextAreaPrintable(text, pj.defaultPage()));

		try {
			if (pj.printDialog(aset)) {
				pj.print();
			}
		} catch (PrinterException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static boolean isValidName(String name) {
		if (!Character.isJavaIdentifierStart(name.charAt(0)))
			return false;
		for (int i = 1; i < name.length(); i++)
			if (!Character.isJavaIdentifierPart(name.charAt(i)))
				return false;
		return true;
	}

	/**
	 * Diese Methode initialisiert eine Action mit den Daten aus dem
	 * ResourceBundle.
	 * 
	 * @param action
	 *            Die zu initialisierende Action
	 * @param key
	 *            Der ue aus dem ResourceBundle
	 */
	public static void setData(Action action, String key) {
		ResourceBundle resources = Utils.getResources();
		action.putValue(Action.NAME, resources.getString(key + ".text"));
		action.putValue(Action.SMALL_ICON, Utils.getIcon(resources
				.getString(key + ".icon")));
		action.putValue(Action.MNEMONIC_KEY, new Integer(resources.getString(
				key + ".mnemonic").charAt(0)));
		try {
			action.putValue(Action.SHORT_DESCRIPTION, resources.getString(key
					+ ".tooltip"));
			action.putValue(Action.ACCELERATOR_KEY, KeyStroke
					.getKeyStroke(resources.getString(key + ".keystroke")));
		} catch (MissingResourceException e) {
		}
	}

	public static JFileChooser getFileChooser() {
		JFileChooser fc = new JFileChooser(new HamsterFileSystemView());
		fc.setAcceptAllFileFilterUsed(false);
		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fc.setMultiSelectionEnabled(false);
		fc.setFileView(new HamsterFileView());
		return fc;
	}

	/**
	 * Diese Methode erzeugt einen Button zu einer Action. Dieser hat dann das
	 * Design, das fuer den Einsatz in einer Toolbar ausgerichtet ist.
	 * 
	 * @param action
	 *            Die Action, zu der der Knopf erzeugt wird
	 * @return Der erzeugte Knopf
	 */
	public static JButton createButton(Action action) {
		JButton b = new JButton(action);
		b.setText(null);
		b.setMnemonic(0);
		b.setMargin(TOOLBAR_MARGIN);
		return b;
	}

	/**
	 * Diese Methode erzeugt einen ToggleButton zu einer Action. Dieser hat dann
	 * das Design, das fuer den Einsatz in einer Toolbar ausgerichtet ist.
	 * 
	 * @param action
	 *            Die Action, zu der der Knopf erzeugt wird
	 * @return Der erzeugte Knopf
	 */
	public static JToggleButton createToggleButton(Action action) {
		JToggleButton b = new JToggleButton(action);
		b.setText(null);
		b.setMnemonic(0);
		b.setMargin(TOOLBAR_MARGIN);
		return b;
	}

	/**
	 * Diese Methode erzeugt einen Menueeintrag zu einer Action.
	 * 
	 * @param action
	 *            Die Action, zu der der Menueeintrag erzeugt wird
	 * @return Der erzeugte Menueeintrag
	 */
	public static JMenuItem createMenuItem(String text, String icon,
			String command, String shortcut) {
		JMenuItem i = new JMenuItem(text, getIcon(icon));
		i.setActionCommand(command);
		i.setAccelerator(KeyStroke.getKeyStroke(shortcut));
		return i;
	}

	/**
	 * Diese Methode liefert das Image zu dem uebergebenen Namen. Alle Bilder
	 * liegen in dem Verzeichnis resources.
	 * 
	 * @param name
	 *            Der Name der Datei, aus der das Bild gelesen werden soll.
	 * @return Das Image
	 */
	public static Image getImage(String name) {
		// TODO: Media Tracker?
		URL url = ClassLoader.getSystemResource("resources/" + name);
		if (url == null) {
			url = Utils.class.getClassLoader().getResource("resources/" + name);
		}
		return Toolkit.getDefaultToolkit().createImage(url);
	}

	/**
	 * Diese Methode erzeugt ein Icon aus dem Aufruf der Methode getImage.
	 * 
	 * @param name
	 *            Der Name der Datei, aus der das Bild gelesen werden soll.
	 * @return Das ImageIcon.
	 */
	public static ImageIcon getIcon(String name) {
		return new ImageIcon(getImage(name));
	}

	/**
	 * Diese Methode stellt sicher, dass die Standard-Verzeichnisstruktur
	 * vorhanden ist, in der die Hamsterprorgramme und -territorien gespeichert
	 * werden.
	 */
	public static void ensureHome() {
		File file = new File(HOME);
		if (!file.exists())
			file.mkdir();
	}

	public static boolean ask(Component c, String key) {
		return JOptionPane.showConfirmDialog(c, Utils.getResource(key), Utils
				.getResource(key + ".title"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION;
	}

	public static boolean ask(Component c, String key, String param) {
		return JOptionPane.showConfirmDialog(c, Utils.getResource(key, param),
				Utils.getResource(key + ".title"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION;
	}

	public static int confirm(Component c, String key) {
		return JOptionPane.showConfirmDialog(c, Utils.getResource(key), Utils
				.getResource(key + ".title"), JOptionPane.YES_NO_CANCEL_OPTION); // dibo
	}

	public static String input(Component c, String key) {
		return JOptionPane.showInputDialog(c, Utils.getResource(key), Utils
				.getResource(key + ".title"), JOptionPane.QUESTION_MESSAGE);
	}

	public static String input(Component c, String key, String value) {
		return (String) JOptionPane.showInputDialog(c, Utils.getResource(key),
				Utils.getResource(key + ".title"),
				JOptionPane.QUESTION_MESSAGE, null, null, value);
	}

	public static void message(Component c, String key) {
		JOptionPane.showMessageDialog(c, Utils.getResource(key));
	}

	public static void message(Component c, String key, String param) {
		JOptionPane.showMessageDialog(c, Utils.getResource(key, param));
	}

	// dibo

	public static boolean security = true;

	public static void loadProperties() {
		try {
			Properties p = new Properties();
			p.load(new FileInputStream("hamster.properties"));
			String str = p.getProperty("security");
			if (str != null && str.equals("false")) {
				Utils.security = false;
			}
			str = p.getProperty("home");
			if (str != null) {
				Utils.HOME = str;
			}
		} catch (Exception exc) {
		}

	}
}